/*
 *	coretrace - basic debugger for embedded Linux
 *	Copyright (C) 2004 Ronny L Nilsson
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 *
 */



/***********************************************************
 * x86 specific corefile analyser
 *
 **********************************************************/


#include <sys/types.h>
#include <asm/string.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/procfs.h>

#include "main.h"
#include "core-linux.h"
#include "exe-elf.h"




//====== DEFINITIONS =================


//====== GLOBALS ==================
extern struct core_t *core;


//====== PROTOTYPES ================
int coreArchStackTrace(void);
int coreArchDefLinkRegion(void);



//----------------------------------------------------------
// Return a human readable string, telling breifely
// what's in the corefile and what was the malfunction
// creating it.
//----------------------------------------------------------
int coreGetArchSummary(void) {
	int errLoc, res;
	
	errLoc=0;

	/*if(core && core->prstatus) {
		int temp1;
		for(temp1=0; temp1<ELF_NGREG; temp1++) {
			DBG("prreg %d 0x%lx\n", temp1, 
				core->prstatus->pr_reg[temp1]);
		}
	}*/
	
	if(core && core->regs) {
		res=snprintf(core->summary + core->summaryLength, 
			MAX_SUMMARY_LENGTH - core->summaryLength, 
			"Crash at addr: 0x%8.8lx\n", core->regs->eip);
		if(res>0) core->summaryLength+=res;
	}
	
	if(!errLoc) errLoc=coreArchStackTrace();
	
	return errLoc;
}




//----------------------------------------------------------
// Locate the stack memory area in corefie
//----------------------------------------------------------
int coreArchStackLocate(void) {
	int errLoc;
	struct map_t *map;
	
	errLoc=0;
		
	map=core->maps;
	while(!errLoc && map && !core->stack) {
		if(map->hdr->p_vaddr <= core->regs->esp && 
				map->hdr->p_vaddr + map->hdr->p_memsz >= core->regs->esp) {
			//DBG("Stack found in mapping 0x%8.8x\n", map->hdr->p_vaddr);
			core->stack=map;
			core->stack->name=malloc(strlen(stackName)+1);
			if(core->stack->name) strcpy(core->stack->name, stackName);			
		}
		map=map->next;
	}
		
	return errLoc;
}




//----------------------------------------------------------
// Do stack backtrace
//----------------------------------------------------------
int coreArchStackTrace(void) {
	int errLoc, frameCnt, res;
	struct map_t *map;
	long frameOffs;
	long ebp, eip;
	
	errLoc=0;
	frameCnt=0;
	map=NULL;
		
	if(core && core->regs && core->stack) {
		ebp = core->regs->ebp;
		eip = core->regs->eip;
		while(!errLoc && ebp) {
			DBG("Stackframe %d at 0x%8.8lx\n", frameCnt, ebp);
			if(eip) {
				res=snprintf(core->summary + core->summaryLength, 
					MAX_SUMMARY_LENGTH - core->summaryLength, 
					"Backtrace: #%d 0x%8.8lx", frameCnt, eip);
				if(res>0) core->summaryLength+=res;

				/* Find out which memmap this instruction
				 * was executed in. */
				map=core->maps;
				while(!errLoc && map) {
					if(eip >= map->hdr->p_vaddr && 
							eip <= map->hdr->p_vaddr + map->hdr->p_memsz && 
							map->hdr->p_flags & PF_X) {
						break;
					}
					map=map->next;
				}			
				if(map) {
					if(map->isDynamic) {
						res=snprintf(core->summary + core->summaryLength, 
							MAX_SUMMARY_LENGTH - core->summaryLength, 
							" (0x%8.8lx relocatable)", eip - map->hdr->p_vaddr);
						if(res>0) core->summaryLength+=res;
					}
					 if(map->name) {
						res=snprintf(core->summary + core->summaryLength, 
							MAX_SUMMARY_LENGTH - core->summaryLength, 
							" in %s", map->name);
						if(res>0) core->summaryLength+=res;
					}
				}
				res=snprintf(core->summary + core->summaryLength, 
					MAX_SUMMARY_LENGTH - core->summaryLength, "\n");
				if(res>0) core->summaryLength+=res;
			}			
			
			/* Calculate where to find next frame. */
			frameOffs = ebp - core->stack->hdr->p_vaddr + 
				core->stack->hdr->p_offset;
			if(frameOffs+sizeof(long) < core->fileSize) {
				ebp = *((long*) (core->data + frameOffs));
				eip = *(((long*) (core->data + frameOffs)) + 1);
				frameCnt++;
			}
			else {
				fprintf(stderr, "Error, strange stack!\n");
				errLoc=1;
			}
		}
	}

	return errLoc;
}



