/*
 *	coretrace - basic debugger for embedded Linux
 *	Copyright (C) 2004 Ronny L Nilsson
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 *
 */



#ifndef CORE_LINUX_H
#define CORE_LINUX_H

#include <linux/limits.h>
#include <stdint.h>
#include <elf.h>

#if defined(__CRIS__) || defined(__i386__)						// Register descriptors for cris, i386
#include <sys/user.h>
#else															// Register descriptors for ppc
#include <asm/ptrace.h>
#endif



//====== DEFINITIONS =================
#define MAX_SUMMARY_LENGTH	(1024*16)							// Create a summary of this lengt maximum	


#define roundup(x, y)  ((((x)+((y)-1))/(y))*(y))				// Kernel do paddings	


#ifndef NT_TASKSTRUCT
#define NT_TASKSTRUCT	4										// Seems to be missing in cris port of glibc?		
#endif


#if defined(__CRIS__) || defined(__i386__)
typedef struct user_regs_struct user_regs_struct_t;				// Register descriptors for cris, i386		
#else
typedef struct pt_regs user_regs_struct_t;						// Register descriptors for ppc
#endif


struct map_t {													// Memmory mappings
	Elf32_Phdr *hdr;											// Header with addresses		
	char *name;													// String description of whats the map for	
	int isDynamic;												// True if linked dynamicaly			
	struct map_t *next;											// Linked list					
};


struct core_t {													// Corefile descriptor	
	char path[PATH_MAX];
	int fd;														// Open file descr
	uint32_t fileSize;
	
	void *data;													// Memmory maped file, read only	
	uint32_t dataLength;										// Length of maped aread	
	
	Elf32_Ehdr *elf;											// Elf header of file
	Elf32_Phdr *prgHdr;											// Elf program headers	
	Elf32_Nhdr *notes;											// Elf program header notes	
	Elf32_Word notesLength;										// Length of notes section	
	struct elf_prstatus *prstatus;
	struct elf_prpsinfo *psinfo;
//struct user_regs_struct *regs;								// cpu registers when app crashed			
user_regs_struct_t *regs;								// cpu registers when app crashed			
	struct map_t *maps;											// Memmory mappings the crashed app had		
	struct map_t *stack;										// Pointer to the memmap which has been identified as the stack	

	char summary[MAX_SUMMARY_LENGTH];							// Description of what the corefile tell went wrong.	
	long summaryLength;											// String-length of description				
};




//====== GLOBALS ==================
extern long pageSize;											// Systems PAGE_SIZE	
extern const char *stackName;									// Description of the stack memmap			
extern struct core_t *core;										// Descriptor of the corefile		


//====== PROTOTYPES ================
int coreInit(void);
int coreOpen(const char*);
int coreClose(void);
char* coreGetSummary(void);

#endif // CORE_LINUX_H



