/*
 *	coretrace - basic debugger for embedded Linux
 *	Copyright (C) 2004 Ronny L Nilsson
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 *
 */



/***********************************************************
 * PowerPC specific corefile analyser
 *
 **********************************************************/


#include <sys/types.h>
#include <asm/string.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/procfs.h>
#include <asm/ptrace.h>

#include "main.h"
#include "core-linux.h"
#include "exe-elf.h"




//====== DEFINITIONS =================


//====== GLOBALS ==================
extern struct core_t *core;


//====== PROTOTYPES ================
int coreArchStackTrace(void);
int coreArchDefLinkRegion(void);



//----------------------------------------------------------
// Return a human readable string, telling breifely
// what's in the corefile and what was the malfunction
// creating it.
//----------------------------------------------------------
int coreGetArchSummary(void) {
	int res;
	
	/* if(core && core->regs) {
		int temp1;
		for(temp1=0; temp1<32; temp1++) {
			DBG("gpr %d 0x%lx\n", temp1, 
				core->regs->gpr[temp1]);
		}
	} */
	
	if(core && core->regs) {
		res=snprintf(core->summary + core->summaryLength, 
			MAX_SUMMARY_LENGTH - core->summaryLength, 
			"Crash at addr: 0x%8.8lx\n", core->regs->nip);
		if(res>0) core->summaryLength+=res;
	
		coreArchStackTrace();
	}
	
	return 0;
}




//----------------------------------------------------------
// Locate the stack memory area in corefie
//----------------------------------------------------------
int coreArchStackLocate(void) {
	struct map_t *map;
	
	map=core->maps;
	while(map && !core->stack) {
		if(map->hdr->p_vaddr <= core->regs->gpr[PT_R1] && 
				map->hdr->p_vaddr + map->hdr->p_memsz >= core->regs->gpr[PT_R1]) {
			DBG("Stack found in mapping 0x%8.8x\n", map->hdr->p_vaddr);
			core->stack=map;
			core->stack->name=malloc(strlen(stackName)+1);
			if(core->stack->name) strcpy(core->stack->name, stackName);			
		}
		map=map->next;
	}
		
	return 0;
}




//----------------------------------------------------------
// Do stack backtrace
//----------------------------------------------------------
int coreArchStackTrace(void) {
	int errLoc, frameCnt, res;
	struct map_t *map;
	long frameOffs;
	long framep, retp;
	
	errLoc=0;
	frameCnt=0;
	map=NULL;
	
	if(core && core->regs && core->stack) {
		framep = core->regs->gpr[PT_R1];
		retp = 0;
		
		while(!errLoc && framep) {
			DBG("Stackframe %d at 0x%8.8lx\n", frameCnt, framep);
			if(retp) {
				res=snprintf(core->summary + core->summaryLength, 
					MAX_SUMMARY_LENGTH - core->summaryLength, 
					"Backtrace: #%d 0x%8.8lx", frameCnt-1, retp);				// First frame not used by ppc,
				if(res>0) core->summaryLength+=res;								//  so decrease by 1.				

				/* Find out which memmap this instruction
				 * was executed in. */
				map=core->maps;
				while(!errLoc && map) {
					if(retp >= map->hdr->p_vaddr && 
							retp <= map->hdr->p_vaddr + map->hdr->p_memsz && 
							map->hdr->p_flags & PF_X) {
						break;
					}
					map=map->next;
				}			
				if(map) {
					if(map->isDynamic) {
						res=snprintf(core->summary + core->summaryLength, 
							MAX_SUMMARY_LENGTH - core->summaryLength, 
							" (0x%8.8lx relocatable)", retp - map->hdr->p_vaddr);
						if(res>0) core->summaryLength+=res;
					}
					 if(map->name) {
						res=snprintf(core->summary + core->summaryLength, 
							MAX_SUMMARY_LENGTH - core->summaryLength, 
							" in %s", map->name);
						if(res>0) core->summaryLength+=res;
					}
				}
				res=snprintf(core->summary + core->summaryLength, 
					MAX_SUMMARY_LENGTH - core->summaryLength, "\n");
				if(res>0) core->summaryLength+=res;
			}
			
			/* Calculate where to find next frame. */
			frameOffs = framep - core->stack->hdr->p_vaddr + 
				core->stack->hdr->p_offset;
			if(frameOffs+sizeof(long) < core->fileSize) {
				framep = *((long*) (core->data + frameOffs));
				if(frameCnt) {								
					retp = *(((long*) (core->data + frameOffs)) + 1);
				}
				else {
					retp = core->regs->nip;
				}
				frameCnt++;
			}
			else {
				fprintf(stderr, "Error, strange stack!\n");
				errLoc=1;
			}
		}
	}
	return errLoc;
}



