/*
 *	coretrace - basic debugger for embedded Linux
 *	Copyright (C) 2004 Ronny L Nilsson
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 *
 */



#ifndef EXE_ELF_H
#define EXE_ELF_H

#include <linux/limits.h>
#include <stdint.h>
#include <elf.h>
#include "core-linux.h"

//====== DEFINITIONS =================

struct exe_t {													// Binary executable descriptor		
	char path[PATH_MAX];
	int fd;														// Open file descr
	uint32_t fileSize;	
	
	void *data;													// Memmory maped file, read only	
	uint32_t dataLength;										// Length of maped aread	
	Elf32_Ehdr *elf;											// Elf header of file
	Elf32_Phdr *prgHdr;											// Elf program headers	
	struct map_t *maps;											// Memmory mappings the appa requests	
	
	int isDynamic;												// True if executable is linked dynamicaly	
};


//====== GLOBALS ==================
extern struct exe_t *exe;										// Descriptor of the binary who crached		



//====== PROTOTYPES ================
int exeOpen(const char*);
int exeClose(void);
int exeGetSummary(void);

#endif // EXE_ELF_H


